#include "elemswitchc.h"
#include "globalc.h"

/**
   function computes coupling matrix of particular elements
   
   @param eid - element id
   @param lcid - load case id
   @param vm - coupling matrix of one element
   
   JK, 28.7.2001
*/
void upper_cond_coupl_mat (long eid,long lcid,matrix &vm)
{
  elemtypec te;
  
  te=Ct->give_elem_type (eid);
  
  switch (te){
  case coupbar:{
    Cbar->res_upper_cond_coup_matrix (eid,vm);
    break;
  }
  case coupquad:{
    Cquad->res_upper_cond_coup_matrix (eid,vm);
    break;
  }
  case couphex:{
    Chex->res_upper_cond_coup_matrix (eid,vm);
    break;
  }
  default:{
    fprintf (stderr,"\n\n unknown element type is required in function");
    fprintf (stderr,"\n upper_cond_coupl_mat (file %s, line %d).\n",__FILE__,__LINE__);
  }
  }
  
}

/**
   function computes coupling matrix of particular elements
   
   @param eid - element id
   @param lcid - load case id
   @param vm - coupling matrix of one element

   JK, 28.7.2001
*/
void lower_cond_coupl_mat (long eid,long lcid,matrix &vm)
{
  elemtypec te;
  
  te=Ct->give_elem_type (eid);
  
  switch (te){
  case coupbar:{
    Cbar->res_lower_cond_coup_matrix (eid,vm);
    break;
  }
  case coupquad:{
    Cquad->res_lower_cond_coup_matrix (eid,vm);
    break;
  }
  case couphex:{
    Chex->res_lower_cond_coup_matrix (eid,vm);
    break;
  }
  default:{
    fprintf (stderr,"\n\n unknown element type is required in function");
    fprintf (stderr,"\n lower_cond_coupl_mat (file %s, line %d).\n",__FILE__,__LINE__);
  }
  }
  
}

/**
   function computes coupling matrix of particular elements
   
   @param eid - element id
   @param lcid - load case id
   @param vm - coupling matrix of one element
   
   JK, 28.7.2001
*/
void upper_cap_coupl_mat (long eid,long lcid,matrix &vm)
{
  elemtypec te;
  
  te=Ct->give_elem_type (eid);
  
  switch (te){
  case coupbar:{
    Cbar->res_upper_cap_coup_matrix (eid,vm);
    break;
  }
  case coupquad:{
    Cquad->res_upper_cap_coup_matrix (eid,vm);
    break;
  }
  case couphex:{
    Chex->res_upper_cap_coup_matrix (eid,vm);
    break;
  }
  default:{
    fprintf (stderr,"\n\n unknown element type is required in function");
    fprintf (stderr,"\n upper_cap_coupl_mat (file %s, line %d).\n",__FILE__,__LINE__);
  }
  }
  
}

/**
   function computes coupling matrix of particular elements
   
   @param eid - element id
   @param lcid - load case id
   @param vm - coupling matrix of one element
   
   JK, 28.7.2001
*/
void lower_cap_coupl_mat (long eid,long lcid,matrix &vm)
{
  elemtypec te;
  
  te=Ct->give_elem_type (eid);
  
  switch (te){
  case coupbar:{
    Cbar->res_lower_cap_coup_matrix (eid,vm);
    break;
  }
  case coupquad:{
    Cquad->res_lower_cap_coup_matrix (eid,vm);
    break;
  }
  case couphex:{
    Chex->res_lower_cap_coup_matrix (eid,vm);
    break;
  }
  default:{
    fprintf (stderr,"\n\n unknown element type is required in function");
    fprintf (stderr,"\n lower_cap_coupl_mat (file %s, line %d).\n",__FILE__,__LINE__);
  }
  }
  
}

/**
   function computes values on element int. points on coupling mesh
   
   JK, 28.7.2001
*/
void intpointvaluesc (long eid)
{
  elemtypec te;
  
  te=Ct->give_elem_type (eid);

  switch (te){
  case coupbar:{
    Cbar->intpointval (eid);
    break;
  }
  case coupquad:{
    //Cquad->intpointval (eid);
    break;
  }
  case couphex:{
    //Chex->intpointval (eid);
    break;
  }
  default:{
    fprintf (stderr,"\n\n unknown element type is required in function");
    fprintf (stderr,"\n intpointvaluesc (file %s, line %d).\n",__FILE__,__LINE__);
  }
  }
  
}

/**
   function computes strains on element int. points on coupling mesh
   
   JK, 28.7.2001
*/
void intpointstrainsc (long eid)
{
  elemtypec te;
  
  te=Ct->give_elem_type (eid);

  switch (te){
  case coupbar:{
    Cbar->res_mainip_strains (eid);
    break;
  }
  case coupquad:{
    //Cquad->res_mainip_strains (eid);
    break;
  }
  case couphex:{
    //Chex->res_mainip_strains (eid);
    break;
  }
  default:{
    fprintf (stderr,"\n\n unknown element type is required in function");
    fprintf (stderr,"\n intpointstrainsc (file %s, line %d).\n",__FILE__,__LINE__);
  }
  }

  //  function modifies strains
  //  eigenstrains are subtracted from the total strains
  Cmu->eigstrmod ();
}

/**
   function computes gradients on element int. points on coupling mesh
   
   JK, 28.7.2001
*/
void intpointgradientsc (long eid)
{
  elemtypec te;
  
  te=Ct->give_elem_type (eid);

  switch (te){
  case coupbar:{
    Cbar->intpointgrad (eid);
    break;
  }
  case coupquad:{
    //Cquad->intpointgrad (eid);
    break;
  }
  case couphex:{
    //Chex->intpointgrad (eid);
    break;
  }
  default:{
    fprintf (stderr,"\n\n unknown element type is required in function");
    fprintf (stderr,"\n intpointgradientsc (file %s, line %d).\n",__FILE__,__LINE__);
  }
  }
  
}

/**
   function assembles vector of coupling right hand side
   
   @param lcid - load case id
   @param rhs - right hand side
   
   21.3.2004, JK+TKr
*/
void assemble_coup (long lcid,double *rhs,long n)
{
  long i,mndofe,*cn,ne;
  vector lv;
  elemtypec te;
  
  ne = Ct->ne;
  
  nullv (rhs,n);
  
  //  contributions from conduction
  for (i=0;i<ne;i++){
    te=Ct->give_elem_type (i);
    mndofe=Gtm->give_ndofe (i);
    allocv (mndofe,lv);
    fillv (0.0,lv);

    switch (te){
    case coupbar:{
      Cbar->res_upper_cond_coup_vector (lv,i);
      break;
    }
    case coupquad:{
      Cquad->res_upper_cond_coup_vector (lv,i);
      break;
    }
    case couphex:{
      Chex->res_upper_cond_coup_vector (lv,i);
      break;
    }
    default:{
      fprintf (stderr,"\n\n unknown element type is required in function");
      fprintf (stderr,"\n assemble_coup (file %s, line %d).\n",__FILE__,__LINE__);
    }
    }
    
    cn = new long [mndofe];
    Mt->give_code_numbers (i,cn);
    locglob (rhs,lv.a,cn,mndofe);
    destrv (lv);
    delete [] cn;
  }  
}



/**
   function copies coupling data to MEFEL
   
   11/05/2010, TKr
*/
void metr_mefel ()
{
  long i;
  elemtypec te;
  
  for (i=0;i<Ct->ne;i++){
    te = Ct->give_elem_type (i);
    
    switch (te){
    case coupbar:{
      Cbar->metr_mefel (i);
      break;
    }
    case coupquad:{
      //Cquad->metr_mefel (i);
      break;
    }
    case coupaxiquad:{
      //Caxiq->metr_mefel (i);
      break;
    }
    case couphex:{
      //Chex->metr_mefel (i);
      break;
    }
    default:{
      fprintf (stderr,"\n\n unknown type of element is required in function mefel_metr (file %s, line %d).\n",__FILE__,__LINE__);
    }
    }
  }
}


/**
   function copies mechanical data to METR
   
   JK, 29.10.2004
*/
void mefel_metr ()
{
  long i;
  elemtypec te;
  
  for (i=0;i<Ct->ne;i++){
    te = Ct->give_elem_type (i);
    
    switch (te){
    case coupbar:{
      Cbar->mefel_metr (i);
      break;
    }
    case coupquad:{
      Cquad->mefel_metr (i);
      break;
    }
    case coupaxiquad:{
      Caxiq->mefel_metr (i);
      break;
    }
    default:{
      fprintf (stderr,"\n\n unknown type of element is required in function mefel_metr (file %s, line %d).\n",__FILE__,__LINE__);
    }
    }
  }
}



/**
   function copies mechanical data to METR
   
   JK, 29.10.2004
*/
  void trfel_metr ()
{
  long i;
  elemtypec te;
  
  for (i=0;i<Ct->ne;i++){
    te = Ct->give_elem_type (i);
    
    switch (te){
    case coupbar:{
      Cbar->trfel_metr (i);
      break;
    }
    case coupquad:{
      Cquad->trfel_metr (i);
      break;
    }
    case coupaxiquad:{
      Caxiq->trfel_metr (i);
      break;
    }
    default:{
      fprintf (stderr,"\n\n unknown type of element is required in function trfel_metr (file %s, line %d).\n",__FILE__,__LINE__);
    }
    }
  }
}


/**
   function computes strains caused by coupling terms
   
   JK, 31.10.2004
*/
void trfel_mefel ()
{
  long i;
  elemtypec te;
  
  for (i=0;i<Ct->ne;i++){
    te = Ct->give_elem_type (i);
    
    switch (te){
    case coupquad:{
      Cquad->trfel_mefel (i);
      break;
    }
    case coupaxiquad:{
      Caxiq->trfel_mefel (i);
      break;
    }
    default:{
      fprintf (stderr,"\n\n unknown type of element is required in function trfel_mefel (file %s, line %d).\n",__FILE__,__LINE__);
    }
    }
  }
  
}
