/*
    File:             onemediumc.cpp
    Author:           Tomas Krejci
    Purpose:          computes conductivity and capacity matrices in a material point for coupled three media trasport with mechanics
*/ 

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "aliast.h"
#include "onemediumc.h"
#include "coupmatu.h"
#include "coupmatl.h"
#include "globalc.h"
#include "global.h"
#include "globalt.h"
#include "isotrmatc.h"
#include "intpoints.h"

medc1::medc1()
{  
  /* scale_t = Tp->scale1;
     scale_u = Mp->scale;
  */
  //not completed
  scale_t = 1.0;
  scale_u = 1.0;
}
medc1::~medc1()
{}


/**
   function computes conductivity matrix D in a material point for one medium transfer
   for upper block
   
   @param d   - conductivity %matrix
   @param ri  - row index
   @param ci  - column index
   @param ipp - number of integration point

*/
void medc1::matcond_u (matrix &d,long ri,long ci,long ipp)
{
  long i,m;
  double k;
  m = d.m;
  
  i = Cmu->ip[ipp].idm;

  switch (Cp->mednam){//names of transported media
  case mech_water:
  case mech_heat:{
    
    switch (Cmu->ip[ipp].tm){//material type
    case isotransmatc:{
      matrix sd(d.m,d.n);
      //t  = Cmu->ip[ipp].av[0]*scale_t;//scaling
      k = Cmu->itrmc[i].get_alpha();//*scale_u;//scaling
      k = -1.0*k;

      switch (m){
      case 1:{//1D
	fillm(0.0,sd);

	sd[0][0] = k;
	break;
      }
      case 3:{//2D
	fillm(0.0,sd);

	sd[0][0] = k;
	sd[1][0] = k;
	sd[2][0] = 0.0;  
	break;
      }
      case 6:{//3D
	fillm(0.0,sd);

	sd[0][0]=k;  
	sd[1][0]=k;   
	sd[2][0]=k;
	sd[3][0]=0.0;   
	sd[4][0]=0.0;  
	sd[5][0]=0.0;
	break;
      }
      default:{
	print_err("\n Unknown number of components of stress tensor is required in ", __FILE__, __LINE__, __func__);
      }
      }
      matrix s(d.m,d.m);
      
      Cmu->matstiff (s,ipp);//only for temperature
      mxm(s,sd,d);
      
      destrm (s); destrm (sd);

      break;
    }
    case sejtkrc:{
      double pw;
      
      pw = Tm->ip[ipp].av[0];
      k = Cmu->sejtkrmc[i].get_kuw(pw);//*scale_u;//scaling

      switch (m){
      case 1:{//1D
	d[0][0] = k;
	break;
      }
      case 3:{//2D
	d[0][0] = k;
	d[1][0] = k;
	d[2][0] = 0.0;  
	break;
      }
      case 6:{//3D
	d[0][0]=k;  
	d[1][0]=k;   
	d[2][0]=k;
	d[3][0]=0.0;   
	d[4][0]=0.0;  
	d[5][0]=0.0;
	break;
      }
      default:{
	print_err("\n Unknown number of components of stress tensor is required in ", __FILE__, __LINE__, __func__);
      }
      }
      break;
    }
    default:{
      print_err("\n Unknown material type is required in ", __FILE__, __LINE__, __func__);
    }
    }  
    break;
  }
  default:{
    print_err("\n Unknown media name is required in ", __FILE__, __LINE__, __func__);
  }
  }
}




/**
   function computes capacity matrix D in a material point for one medium transfer
   for upper block
   
   @param d   - cpapcity %matrix
   @param ri  - row index
   @param ci  - column index
   @param ipp - number of integration point

*/
void medc1::matcap_u (matrix &d,long ri,long ci,long ipp)
{
  long i,m;
  double c;
  m = d.m;
  
  i = Cmu->ip[ipp].idm;

  switch (Cp->mednam){//names of transported media
  case mech_water:
  case mech_heat:{
    
    switch (Cmu->ip[ipp].tm){//material type
    case isotransmatc:{
      //t  = Cmu->ip[ipp].av[0]*scale_t;//scaling
      
      c = 0.0;
      
      break;
    }
    case sejtkrc:{
      double pw;
      
      pw = Tm->ip[ipp].av[0];
      c = Cmu->sejtkrmc[i].get_capuw(pw);//*scale_u;//scaling
      break;
    }
    default:{
      print_err("\n Unknown material type is required in ", __FILE__, __LINE__, __func__);
    }
    }  
    
    switch (m){
    case 1:{//1D
      fillm(0.0,d);
      
      d[0][0] = c;
      break;
    }
    case 3:{//2D
      fillm(0.0,d);
      
      d[0][0] = c;
      d[1][0] = c;
      d[2][0] = 0.0;  
      break;
    }
    case 6:{//3D
      fillm(0.0,d);
      
      d[0][0]=c;  
      d[1][0]=c;   
      d[2][0]=c;
      d[3][0]=0.0;   
      d[4][0]=0.0;  
      d[5][0]=0.0;
      break;
    }
    default:{
      print_err("\n Unknown number of components of stress tensor is required in ", __FILE__, __LINE__, __func__);
    }
    }
      
    break;
  }
  default:{
    print_err("\n Unknown media name is required in ", __FILE__, __LINE__, __func__);
  }
  }
}


/**
   function computes conductivity matrix D in a material point for one medium transfer
   for lower block
   
   @param d   - conductivity %matrix
   @param ri  - row index
   @param ci  - column index
   @param ipp - number of integration point

*/
void medc1::matcond_l (matrix &d,long ri,long ci,long ipp)
{
  long i,n;
  double k;
  n = d.n;
  
  i = Cml->ip[ipp].idm;

  switch (Cp->mednam){//names of transported media
  case mech_water:
  case mech_heat:{
    
    switch (Cml->ip[ipp].tm){//material type
    case isotransmatc:{
      //t  = Cml->ip[ipp].av[0]*scale_t;//scaling
      k = 0.0;
      break;
    }
    case sejtkrc:{
      double pw;
      
      pw = Tm->ip[ipp].av[0];
      k = Cmu->sejtkrmc[i].get_kwu(pw);//*scale_u;//scaling
      break;
    }
    default:{
      print_err("\n Unknown material type is required in ", __FILE__, __LINE__, __func__);
    }
    }  
    
    switch (n){
    case 1:{//1D
      fillm(0.0,d);
      
      d[0][0] = k;
      break;
    }
    case 3:{//2D
      fillm(0.0,d);
      
      d[0][0] = k;
      d[0][1] = k;
      d[0][2] = 0.0;  
      break;
    }
    case 6:{//3D
      fillm(0.0,d);
      
      d[0][0]=k;  
      d[0][1]=k;   
      d[0][2]=k;
      d[0][3]=0.0;   
      d[0][4]=0.0;  
      d[0][5]=0.0;
      break;
    }
    default:{
      print_err("\n Unknown number of components of stress tensor is required in ", __FILE__, __LINE__, __func__);
    }
    }
    
    break;
  }
  default:{
    print_err("\n Unknown media name is required in ", __FILE__, __LINE__, __func__);
  }
  }
}




/**
   function computes capacity matrix D in a material point for one medium transfer
   for lower block
   
   @param d   - cpapcity %matrix
   @param ri  - row index
   @param ci  - column index
   @param ipp - number of integration point

*/
void medc1::matcap_l (matrix &d,long ri,long ci,long ipp)
{
  long i,n;
  double c;
  n = d.n;
  
  i = Cml->ip[ipp].idm;

  switch (Cp->mednam){//names of transported media
  case mech_water:
  case mech_heat:{
    
    switch (Cml->ip[ipp].tm){//material type
    case isotransmatc:{
      //t  = Cml->ip[ipp].av[0]*scale_t;//scaling
      
      c = 0.0;
      break;
    }
    case sejtkrc:{
      double pw;
      
      pw = Tm->ip[ipp].av[0];
      c = Cmu->sejtkrmc[i].get_capwu(pw);//*scale_u;//scaling
      break;
    }
    default:{
      print_err("\n Unknown material type is required in ", __FILE__, __LINE__, __func__);
    }
    }  
    
    switch (n){
    case 1:{//1D
      fillm(0.0,d);
      
      d[0][0] = c;
      break;
    }
    case 3:{//2D
      fillm(0.0,d);
      
      d[0][0] = c;
      d[0][1] = c;
      d[0][2] = 0.0;  
      break;
    }
    case 6:{//3D
      fillm(0.0,d);
      
      d[0][0]=c;  
      d[0][1]=c;   
      d[0][2]=c;
      d[0][3]=0.0;   
      d[0][4]=0.0;  
      d[0][5]=0.0;
      break;
    }
    default:{
      print_err("\n Unknown number of components of stress tensor is required in ", __FILE__, __LINE__, __func__);
    }
    }
    break;
  }
  default:{
    print_err("\n Unknown media name is required in ", __FILE__, __LINE__, __func__);
  }
  }
}



/**
   function computes right-hand-side matrix D in a material point for one medium transfer
   for upper block
   
   @param d   - right-hand-side %matrix
   @param ri  - row index
   @param ci  - column index
   @param ipp - number of integration point

*/
void medc1::rhs_u (matrix &d,long ri,long ci,long ipp)
{
  long i,m;
  double k;
  m = d.m;
  
  i = Cmu->ip[ipp].idm;
  
  switch (Cp->mednam){//names of transported media
  case mech_water:
  case mech_heat:{
    
    switch (Cmu->ip[ipp].tm){//material type
    case isotransmatc:{
      matrix sd(d.m,d.n);
      //t  = Cmu->ip[ipp].av[0]*scale_t;//scaling
      
      k = Cmu->itrmc[i].get_alpha();//*scale_u;//scaling
      k = -1.0*k;
      
      switch (m){
      case 1:{//1D
	fillm(0.0,sd);
	
	sd[0][0] = k;
	break;
      }
      case 3:{//2D
	fillm(0.0,sd);
	
	sd[0][0] = k;
	sd[1][0] = k;
	sd[2][0] = 0.0;  
	break;
      }
      case 6:{//3D
	fillm(0.0,sd);
	
	sd[0][0]=k;  
	sd[1][0]=k;   
	sd[2][0]=k;
	sd[3][0]=0.0;   
	sd[4][0]=0.0;  
	sd[5][0]=0.0;
	break;
      }
      default:{
	print_err("\n Unknown number of components of stress tensor is required in ", __FILE__, __LINE__, __func__);
      }
      }
      matrix s(d.m,d.m);
      
      Cmu->matstiff (s,ipp);//only for temperature
      mxm(s,sd,d);
      
      destrm (s); destrm (sd);
      
      break;
    }
    case sejtkrc:{
      double pw;
      
      pw = Tm->ip[ipp].av[0];
      k = Cmu->sejtkrmc[i].get_kuw(pw);//*scale_u;//scaling
      
      switch (m){
      case 1:{//1D
	d[0][0] = k;
	break;
      }
      case 3:{//2D
	d[0][0] = k;
	d[1][0] = k;
	d[2][0] = 0.0;  
	break;
      }
      case 6:{//3D
	d[0][0]=k;  
	d[1][0]=k;   
	d[2][0]=k;
	d[3][0]=0.0;   
	d[4][0]=0.0;  
	d[5][0]=0.0;
	break;
      }
      default:{
	print_err("\n Unknown number of components of stress tensor is required in ", __FILE__, __LINE__, __func__);
      }
      }
      break;
    }
    default:{
      print_err("\n Unknown material type is required in ", __FILE__, __LINE__, __func__);
    }
    }
    break;
  }
  default:{
    print_err("\n Unknown media name is required in ", __FILE__, __LINE__, __func__);
  }
  }
}


/**
   function computes initial values in a material point for one medium transfer
   for upper block
   
   @param ipp - number of integration point

*/
void medc1::initvalues (long ipp)
{
  long i;
  strastrestate sss;
  
  //  type of strain/stress state
  sss = Cmu->ip[ipp].mssst;
  
  switch (Cp->mednam){//names of transported media
  case mech_water:
    break;
  case mech_heat:{
    
    switch (Cmu->ip[ipp].tm){//material type
    case isotransmatc:{
      
      //null temprstrains
      for(i=0;i<sss;i++)
	Cmu->tempstrains[ipp][i] = 0;

      Cmu->inittempr[ipp] = Cmu->ip[ipp].av[0];//inital temperature

      break;
    }
    default:{
      print_err("\n Unknown material type is required in ", __FILE__, __LINE__, __func__);
    }
    }
    break;
  }
  default:{
    print_err("\n Unknown media type is required in ", __FILE__, __LINE__, __func__);
  }
  }
}


/**
   function computes update values in a material point for one medium transfer
   for upper block
   
   @param ipp - number of integration point

*/
void medc1::updatevalues (long ipp)
{
  long i;
  double dt,alpha;
  strastrestate sss;
  
  //  type of strain/stress state
  sss = Cmu->ip[ipp].mssst;
  
  switch (Cp->mednam){//names of transported media
  case mech_water:
    break;
  case mech_heat:{
    
    switch (Cmu->ip[ipp].tm){//material type
    case isotransmatc:{
      
      //null temprstrains
      for(i=0;i<sss;i++)
	Cmu->tempstrains[ipp][i] = 0;

      Cmu->tempr[ipp] = Cmu->ip[ipp].av[0];//actual temperature
      
      break;
    }
    default:{
      print_err("\n Unknown material type is required in ", __FILE__, __LINE__, __func__);
    }
    }
    break;
  }
  default:{
    print_err("\n Unknown media type is required in ", __FILE__, __LINE__, __func__);
  }
  }
}

/**
   function computes update values in a material point for one medium transfer
   for upper block
   
   @param ipp - number of integration point

*/
void medc1::tempstrains (long ipp)
{
  long i;
  double dt,alpha;
  strastrestate sss;
  
  //  type of strain/stress state
  sss = Cmu->ip[ipp].mssst;
  
  switch (Cp->mednam){//names of transported media
  case mech_water:
    break;
  case mech_heat:{
    
    switch (Cmu->ip[ipp].tm){//material type
    case isotransmatc:{
      

      //null temprstrains
      for(i=0;i<sss;i++)
	Cmu->tempstrains[ipp][i] = 0;
      
      //  change of temperature
      dt = Cmu->tempr[ipp]-Cmu->inittempr[ipp];

      i = Cmu->ip[ipp].idm;
      alpha = Cmu->itrmc[i].get_alpha();
      
      switch (sss){
      case bar:{
	Cmu->tempstrains[ipp][0]=alpha*dt;
	break;
      }
      case plbeam:{
	Cmu->tempstrains[ipp][0]=alpha*dt;
	Cmu->tempstrains[ipp][1]=0.0;
	Cmu->tempstrains[ipp][2]=0.0;
	break;
      }
      case planestress:{
	Cmu->tempstrains[ipp][0]=alpha*dt;
	Cmu->tempstrains[ipp][1]=alpha*dt;
	Cmu->tempstrains[ipp][3]=alpha*dt;
	break;
      }
      case planestrain:{
	Cmu->tempstrains[ipp][0]=alpha*dt;
	Cmu->tempstrains[ipp][1]=alpha*dt;
	Cmu->tempstrains[ipp][3]=alpha*dt;
	break;
      }
      case axisymm:{
	Cmu->tempstrains[ipp][0]=alpha*dt;
	Cmu->tempstrains[ipp][1]=alpha*dt;
	Cmu->tempstrains[ipp][2]=alpha*dt;
	break;
      }
      case spacestress:{
	Cmu->tempstrains[ipp][0]=alpha*dt;
	Cmu->tempstrains[ipp][1]=alpha*dt;
	Cmu->tempstrains[ipp][2]=alpha*dt;
	Cmu->tempstrains[ipp][3]=0.0;
	Cmu->tempstrains[ipp][4]=0.0;
	Cmu->tempstrains[ipp][5]=0.0;    
	break;
      }
      default:
	print_err("unknown strain/stress state is required", __FILE__, __LINE__, __func__);
      }
      break;
    }
    default:{
      print_err("\n Unknown material type is required in ", __FILE__, __LINE__, __func__);
    }
    }
    break;
  }
  default:{
    print_err("\n Unknown media type is required in ", __FILE__, __LINE__, __func__);
  }
  }
}
